#include <mach/mach_types.h>

#include <sys/systm.h>
#include <sys/types.h>
#include <sys/conf.h>
#include <miscfs/devfs/devfs.h>
#include <sys/ioctl.h>


// Register device in /dev
static int major = -1;
static void *devfshandle = 0;
static int cdev_open(dev_t dev, int flags, int devtype, struct proc *p) {
    return 0;
}

static int cdev_close(dev_t dev, int flags, int devtype, struct proc *p) {
    return 0;
}

typedef struct transporter {
    int tMInteger;
    char tMChar;
} transporter;

#define KQEMU_INIT          _IOWR('d', 0, u_int32_t)
#define KQEMU_EXEC          _IOWR('d', 1, transporter)
#define KQEMU_GET_VERSION   _IOWR('d', 2, int)

#define KQEMU_VERSION 0x010300
static int cdev_ioctl(dev_t dev, u_long cmd, caddr_t data, int fflag, struct proc *p) {
    int error = 0;
    
    switch(cmd) {
        case KQEMU_INIT: {
            if (data) {
                *(u_int32_t *)data = 102;
                printf("kqemu KEXT: KQEMU_INIT: %d\n", *data);
                break;
            } else {
                printf("kqemu KEXT: KQEMU_INIT: no data\n");
            }
        }
        
        case KQEMU_EXEC: {
            if (data) {
                transporter *tTr = (transporter *)data;
                tTr->tMInteger = 111;
                tTr->tMChar = 'Z';
                printf("kqemu KEXT: KQEMU_EXEC: int=%d char=%c\n", tTr->tMInteger, tTr->tMChar);                
            } else {
                printf("kqemu KEXT: KQEMU_EXEC no data\n");
            }
            break;
        }
        
        case KQEMU_GET_VERSION: {
            if (data) {
            *(int *)data = KQEMU_VERSION;
            printf("kqemu KEXT: KQEMU_VERSION: %08x\n", *(int *)data);
            } else {
                printf("kqemu KEXT: KQEMU_VERSION: no data\n");
            }
            break;
        }
        
        default:
            error = EINVAL;
    }
    
    return error;
}

// Character device switch table
static struct cdevsw chardev = {
	cdev_open, /* open */
	cdev_close, /* close */
	eno_rdwrt, /* read */
	eno_rdwrt, /* write */
	cdev_ioctl, /* ioctl */
	eno_stop, /* stop*/
	eno_reset, /* reset */
	0, /* ttys */
	eno_select, /* select */
	eno_mmap, /* mmap */
	eno_strat, /* strategy */
	eno_getc, /* getc */
	eno_putc, /* putc */
	0 /* type */
};


//starting / stopping kext
kern_return_t kqemu_start (kmod_info_t * ki, void * d) {

    // Register device in /dev
    major = cdevsw_add(major, &chardev);
    if (major == -1) {
        printf("kqemu KEXT: error registering cdevsw\n");
    }
	devfshandle = devfs_make_node(makedev(major, 0), DEVFS_CHAR, UID_ROOT, GID_WHEEL, 0666, "kqemu");
    if (!devfshandle) {
        printf("kqemu KEXT: error creating device node\n");
    }

    printf("kqemu KEXT has loaded!\n");
    return KERN_SUCCESS;
}


kern_return_t kqemu_stop (kmod_info_t * ki, void * d) {

    // remove device from /dev
	devfs_remove(devfshandle);
	cdevsw_remove(major, &chardev);
    
    printf("kqemu KEXT will be unloaded!\n");
    return KERN_SUCCESS;
}




