//
//  controller.m
//  kqemuClient
//
//  Created by Mike Kronenberg on 15.03.07.
//  Copyright 2007 __MyCompanyName__. All rights reserved.
//

#include <sys/ioctl.h>

#import "controller.h"

#define KQEMU_INVALID_FD -1
#define KQEMU_DEVICE "/dev/kqemu"

typedef struct transporter {
    int tMInteger;
    char tMChar;
} transporter;

#define KQEMU_INIT          _IOWR('d', 0, u_int32_t)
#define KQEMU_EXEC          _IOWR('d', 1, transporter)
#define KQEMU_GET_VERSION   _IOWR('d', 2, int)

@implementation controller
- (IBAction)kqemuOpen:(id)sender
{
    kqemu_fd = open(KQEMU_DEVICE, O_RDWR);
    if (kqemu_fd == KQEMU_INVALID_FD) {
        [text setStringValue:[NSString stringWithFormat:@"Could not open '%s'", KQEMU_DEVICE]];
    } else {
        [text setStringValue:@"kqemu opened"];
    }    
}
- (IBAction)kqemuClose:(id)sender
{
    int err;
    err = close(kqemu_fd);
    if (err == KQEMU_INVALID_FD) {
        [text setStringValue:[NSString stringWithFormat:@"Could not close '%s'", KQEMU_DEVICE]];
    } else {
        [text setStringValue:@"kqemu closed"];
    }    
}

- (IBAction)kqemuInit:(id)sender
{
    int ret;
    u_int32_t blockSize = 0;
    ret = ioctl( kqemu_fd, KQEMU_INIT, &blockSize );
    if (ret < 0) {
        [text setStringValue:[NSString stringWithFormat:@"Error %d while initializing QEMU acceleration layer", ret]];
    } else {
        [text setStringValue:[NSString stringWithFormat:@"initialized: %d", blockSize]];
    }
}

- (IBAction)kqemuExec:(id)sender
{
    int ret;
    transporter tTr;
    tTr.tMInteger = 0;
    tTr.tMChar = 'a';
    ret = ioctl(kqemu_fd, KQEMU_EXEC, &tTr);
    if (ret < 0) {
        [text setStringValue:[NSString stringWithFormat:@"Error %d while sending command to QEMU acceleration layer", ret]];
    } else {
        [text setStringValue:[NSString stringWithFormat:@"executed: %d %c", tTr.tMInteger, tTr.tMChar]];
    }
}

- (IBAction)kqemuVersion:(id)sender
{
    int ret;
    int version = 0;
    ret = ioctl(kqemu_fd, KQEMU_GET_VERSION, &version);
    if (ret < 0) {
        [text setStringValue:[NSString stringWithFormat:@"Error %d while getting version of QEMU acceleration layer", ret]];
    } else {
        [text setStringValue:[NSString stringWithFormat:@"version: %08x", version]];
    }
}
@end
